﻿// this is the C# version of the code sample created by Brian A. Randell for the
// MSDN magazine article found here : http://msdn.microsoft.com/en-us/magazine/cc163498.aspx
// all credits go to him for this part of the codebase.
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.VersionControl.Common;


namespace Dialogs
{
    public partial class PicVCLocation : Form
    {
        private const string NODE_PLACEHOLDER = "Hold";
        private bool addNewNodeInProgress = false;

        VersionControlServer _vcs;
        public PicVCLocation(VersionControlServer vcs)
        {
            InitializeComponent();
            _vcs = vcs;
            ServerName = vcs.TeamFoundationServer.Uri.Host;
        }


        public TeamProject[] GetAllProjects()
        {
            return _vcs.GetAllTeamProjects(true);
        }

        public bool LoadProjects()
        {
            TeamProject[] allTeamProjects = GetAllProjects();

            if (allTeamProjects.Length > 0)
            {
                TreeNode n = null;
                try
                {

                    tvProjects.BeginUpdate();
                    foreach (TeamProject tp in allTeamProjects)
                    {
                        n = tvProjects.Nodes.Add(tp.Name);
                        n.Tag = tp.ServerItem;
                        n.Nodes.Add(NODE_PLACEHOLDER);
                    }

                    tvProjects.SelectedNode = tvProjects.Nodes[0];
                    txtSelectedServerPath.Text = tvProjects.SelectedNode.Tag.ToString();
                    btnMakeNewFolder.Enabled = true;
                    btnOK.Enabled = true;
                }
                finally
                {
                    tvProjects.EndUpdate();
                }
                return true;
            }
            else
            {
                return false;
            }
        }

        public string ServerName
        {
            get
            { 
                return lblServer.Text; 
            }
            set
            {

                lblServer.Text = value;
            }
        }

        public string ServerPath
        {
            get { return txtSelectedServerPath.Text; }
        }

        private void AddChildren(TreeNode current)
        {
            string projectName = VersionControlPath.GetTeamProjectName(current.Tag.ToString());

            List<Item> folders = GetFolders(projectName, current.Tag.ToString());
            TreeNode tempNode = null;
            foreach (Item tfsItem in folders)
            {
                tempNode = current.Nodes.Add(VersionControlPath.GetFileName(tfsItem.ServerItem));
                tempNode.Tag = tfsItem.ServerItem;
                tempNode.Nodes.Add(NODE_PLACEHOLDER);
            }
        }


        public List<Item> GetFolders(string projectName, string serverItem)
        {
            TeamProject tp = _vcs.GetTeamProject(projectName);
            ItemSet svrFolders = _vcs.GetItems(serverItem, VersionSpec.Latest,
                RecursionType.OneLevel, DeletedState.NonDeleted, ItemType.Folder);

            List<Item> retval = new List<Item>();
            foreach (Item tfsItem in svrFolders.Items)
            {
                if (!tfsItem.ServerItem.Equals(serverItem,StringComparison.InvariantCulture))
                {
                    retval.Add(tfsItem);
                }
            }
            return retval;
        }
       
        private void MakeNewFolder_Click(object sender, EventArgs e)
        {
            addNewNodeInProgress = true;
            TreeNode selected = tvProjects.SelectedNode;
            TreeNode child = selected.Nodes.Add("New Folder");
            selected.Expand();
            tvProjects.LabelEdit = true;
            child.BeginEdit();
        }

        private void miRefresh_Click(object sender, EventArgs e)
        {
            tvProjects.SelectedNode.Nodes.Clear();
            AddChildren(tvProjects.SelectedNode);
        }

        private void tvProjects_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            e.Node.Tag = e.Node.Parent.Tag.ToString() + "/" + e.Label;
            tvProjects.LabelEdit = false;
            addNewNodeInProgress = false;
            tvProjects.SelectedNode = e.Node;
            txtSelectedServerPath.Text = tvProjects.SelectedNode.Tag.ToString();
        }

        private void tvProjects_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            if (!addNewNodeInProgress)
            {
                if (e.Node.FirstNode.Text == NODE_PLACEHOLDER)
                {
                    try
                    {
                        tvProjects.BeginUpdate();
                        e.Node.Nodes.Clear();
                        AddChildren(e.Node);
                    }
                    finally
                    {
                        tvProjects.EndUpdate();

                    }
                }
            }
        }
        private void tvProjects_Click(object sender, EventArgs e)
        {
            tvProjects.SelectedNode = tvProjects.GetNodeAt(tvProjects.PointToClient(Cursor.Position));
        }

        private void tvProjects_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
                tvProjects.SelectedNode = tvProjects.GetNodeAt(e.X, e.Y);
        }

        private void tvProjects_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            txtSelectedServerPath.Text = e.Node.Tag.ToString();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {

        }

    }
}
