﻿#region Copyright © 2009 Marcel de Vries
/* This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in a
 *    product, an acknowledgment in the product documentation is requested, as
 *    shown here:
 * 
 *    Portions copyright © 2009 Marcel de Vries (http://blogs.infosupport.com/blogs/marcelv).
 * 
 * 2. No substantial portion of this source code may be redistributed without
 *    the express written permission of the copyright holders, where
 *    "substantial" is defined as enough code to be recognizably from this code.
 */
#endregion
using System;
using System.Drawing;
using System.Runtime.InteropServices;
using Microsoft.TeamFoundation.Common;
using Microsoft.VisualStudio.Shell.Interop;

namespace MSDNMagazine.TFSPlugins
{
    //Here the explorer is created in the Team explorer
    [Guid("97CE787C-DE2D-4b5c-AF6D-79E254D83111")]
    public class SubstExplorer : BasicAsyncPlugin
    {
        public SubstExplorer()
         : base(MSDNMagazine.TFSPlugins.SubstExplorerHostPackage.Instance)
        {
        }

        public override String Name
        {
            get { return "Subst drive mappings"; }
        }
        public override int DisplayPriority
        {
            get 
            { 
                // After team explorer build, but before any installed power tools
                // power tools start at 450
                return 400; 
            }
        }
        public override IntPtr OpenFolderIconHandle
        {
            get
            {
                return IconHandle;
            }
        }
        public override IntPtr IconHandle
        {
            get
            {
                return new Bitmap(SubstConfigurationFile.GetCommandImages().Images[2]).GetHicon();
            }
        }
        protected override BaseUIHierarchy GetNewUIHierarchy(IVsUIHierarchy parentHierarchy, uint itemId)
        {
            SubstExplorerUIHierarchy uiHierarchy = new SubstExplorerUIHierarchy(parentHierarchy, itemId, this);
           return uiHierarchy;
        }
        protected override BaseHierarchyNode CreateNewTree(BaseUIHierarchy hierarchy)
        {
            SubstExplorerRoot root = new SubstExplorerRoot(hierarchy.ProjectName + '/' + "SubstExplorerRoot");
            PopulateTree(root);
            // add the tree to the UIHierarchy so it can handle the commands
            if (hierarchy.HierarchyNode == null)
            {
                hierarchy.AddTreeToHierarchy(root, true);
            }
            return root;
        }
        public static void PopulateTree(BaseHierarchyNode teNode)
        {
            string projectName = teNode.CanonicalName.Substring(0, teNode.CanonicalName.IndexOf("/"));
            var substNodes = SubstConfigurationFile.GetMappingsForProject(projectName);
            if (substNodes != null)
            {
                foreach (var substNode in substNodes)
                {
                    SubstExplorerLeaf leafNode = new SubstExplorerLeaf(substNode.name,
                         substNode.drive, substNode.versionControlPath);
                    teNode.AddChild(leafNode);
                }
                // force refresh of icon that changed during add
                if (teNode.IsExpanded)
                {
                    teNode.Expand(false);
                    teNode.Expand(true);
                }
            }
        }    

        
    }
}
