﻿#region Copyright © 2009 Marcel de Vries
/* This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in a
 *    product, an acknowledgment in the product documentation is requested, as
 *    shown here:
 * 
 *    Portions copyright © 2009 Marcel de Vries (http://blogs.infosupport.com/blogs/marcelv).
 * 
 * 2. No substantial portion of this source code may be redistributed without
 *    the express written permission of the copyright holders, where
 *    "substantial" is defined as enough code to be recognizably from this code.
 */
#endregion
using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.TeamFoundation.Common;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;

namespace MSDNMagazine.TFSPlugins
{
    public enum SubstDriveEnum
    {
        E=0,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z
    }
    public class SubstExplorerProperties : ICustomTypeDescriptor, IVsProvideUserContext
    {
        private BaseHierarchyNode m_node = null;
        public SubstExplorerProperties(BaseHierarchyNode node)
        {
            m_node = node;
        }

        #region ICustomTypeDescriptor Members

        public AttributeCollection GetAttributes()
        {
            return AttributeCollection.Empty;
        }

        public string GetClassName()
        {
            return m_node.PropertiesClassName;
        }

        public string GetComponentName()
        {
            return m_node.Name;
        }

        public TypeConverter GetConverter()
        {
            return new TypeConverter();
        }

        public EventDescriptor GetDefaultEvent()
        {
            // no events
            return null;
        }

        public PropertyDescriptor GetDefaultProperty()
        {
            // no default property
            return null;
        }

        public object GetEditor(Type editorBaseType)
        {
            // no custom editor
            return null;
        }

        public EventDescriptorCollection GetEvents(Attribute[] attributes)
        {
            return EventDescriptorCollection.Empty;
        }

        public EventDescriptorCollection GetEvents()
        {
            return EventDescriptorCollection.Empty;
        }

        public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
        {
            List<PropertyDescriptor> list = new List<PropertyDescriptor>();
            PropertyDescriptorCollection descriptors = TypeDescriptor.GetProperties(this, attributes, true);
            for (int i = 0; i < descriptors.Count; i++)
            {
                list.Add(new DesignPropertyDescriptor(descriptors[i]));
            }
            return new PropertyDescriptorCollection(list.ToArray());

        }

        public PropertyDescriptorCollection GetProperties()
        {
            return PropertyDescriptorCollection.Empty;
        }

        public object GetPropertyOwner(PropertyDescriptor pd)
        {
            // return the object implementing the properties
            return this;
        }

        #endregion

        #region IVsProvideUserContext Members

        public int GetUserContext(out IVsUserContext ppctx)
        {
            ppctx = null;
            try
            {
                ppctx = m_node.GetUserContext();
            }
            catch (Exception)
            {
                // failed
                return 1;
            }
            // succeeded
            return 0;
        }

        #endregion
        //properties start here: string _name;
        [Category("Drive mapping")]
        [Description("This represents the path in version control that needs to be maped to the selected drive letter. This path starts with the $/<yourteamproject>/YourBranch/...")]
        [DisplayName("Version Control Path")]
        public string VersionControlPath
        {
            get { return ((SubstExplorerLeaf)m_node).VersionControlPath; }
        }

        [Category("Drive mapping")]
        [Description("Drive letter to use for the subst command. If a drive with the same letter already exists, that drive will first be unsubst.")]
        [DisplayName("Subst drive letter")]
        public SubstDriveEnum SubstDriveLetter
        {
            get { return (SubstDriveEnum)Enum.Parse(typeof(SubstDriveEnum),((SubstExplorerLeaf)m_node).SubstDriveLetter); }
        }

        [Category("Drive mapping")]
        [Description("Display name of the mapping. Must be unique within the team project.")]
        [DisplayName("Mapping name")]
        public string MappingName
        {
            get { return ((SubstExplorerLeaf)m_node).Name; }
        }


    
    }
}
