﻿#region Copyright © 2009 Marcel de Vries
/* This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in a
 *    product, an acknowledgment in the product documentation is requested, as
 *    shown here:
 * 
 *    Portions copyright © 2009 Marcel de Vries (http://blogs.infosupport.com/blogs/marcelv).
 * 
 * 2. No substantial portion of this source code may be redistributed without
 *    the express written permission of the copyright holders, where
 *    "substantial" is defined as enough code to be recognizably from this code.
 */
#endregion
using System;
using System.Diagnostics;
using System.Globalization;

namespace MSDNMagazine.TFSPlugins
{
    internal static class SubstHelper
    {
        #region public methods
        /// <summary>
        /// Returns the list of current virtual drives.
        /// </summary>
        /// <returns>String output from subst.</returns>
        public static string[] SubstedDrives()
        {
            // Splitting the output from subst into seperate strings to get each substed drive into a single string.
            // Because newlines are '\r\n' in Windows, each newline would be an item in the array if the RemoveEmptyEntries is not used.
            return Subst("").Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
        }

        /// <summary>
        /// Associates a path with a drive letter.
        /// </summary>
        /// <returns>String output from subst.</returns>
        public static string Subst(string drive, string directory)
        {
            drive = EnsureDriveStringFormat(drive);
            return Subst(string.Format(CultureInfo.InvariantCulture, "{0} \"{1}\"", drive, directory));
        }

        /// <summary>
        /// Checks if the drive letter is already postfixed with a :
        /// if not it will append it.
        /// </summary>
        /// <param name="drive"></param>
        /// <returns></returns>
        private static string EnsureDriveStringFormat(string drive)
        {
            if (!drive.Contains(":"))
            {
                drive = drive + ":";
            }
            return drive;
        }

        /// <summary>
        /// Deletes a substituted (virtual) drive.
        /// </summary>
        /// <returns>String output from subst.</returns>
        public static string DeleteSubst(string drive)
        {
            drive = EnsureDriveStringFormat(drive);
            return Subst(string.Format(CultureInfo.InvariantCulture, "{0} /D", drive));
        }
        #endregion

        #region private methods
        /// <summary>
        /// Execute subst.
        /// </summary>
        /// <returns>String output from subst.</returns>
        private static string Subst(string arguments)
        {
            ProcessStartInfo pi = new ProcessStartInfo("subst");
            pi.CreateNoWindow = true;
            pi.UseShellExecute = false;
            pi.RedirectStandardOutput = true;
            pi.Arguments = arguments;

            return Process.Start(pi).StandardOutput.ReadToEnd();
        }
        #endregion
   
    }

}
